document.addEventListener('DOMContentLoaded', () => {
    const countrySelect = document.getElementById('countrySelect');
    const stateSelect = document.getElementById('stateSelect');
    const searchInput = document.getElementById('searchInput');
    const tableBody = document.getElementById('tableBody');
    const loader = document.getElementById('loader');

    let countries = [];
    let states = [];
    let ipData = [];

    // Cache for data
    const cache = {
        countries: null,
        states: null,
        ipData: null
    };

    async function fetchData(url) {
        try {
            const response = await fetch(url);
            if (!response.ok) throw new Error('Network response was not ok');
            return await response.json();
        } catch (error) {
            console.error('Error fetching data:', error);
            return null;
        }
    }

    async function initializeData() {
        loader.style.display = 'flex';

        // Fetch all data in parallel
        const [countriesData, statesData, ipDataRaw] = await Promise.all([
            fetchData('https://cscd.api.ipvi.net/json/countries.json'),
            fetchData('https://cscd.api.ipvi.net/json/states.json'),
            fetch('https://raw.githubusercontent.com/axelawh/geofeed/refs/heads/main/location.csv')
                .then(response => response.text())
        ]);

        if (countriesData) {
            cache.countries = countriesData;
            populateCountrySelect(countriesData);
        }

        if (statesData) {
            cache.states = statesData;
        }

        if (ipDataRaw) {
            cache.ipData = parseCSV(ipDataRaw);
            displayResults(cache.ipData.slice(0, 50)); // Show first 50 results initially
        }

        loader.style.display = 'none';
    }

    function parseCSV(csv) {
        return csv.split('\n')
            .filter(line => line.trim())
            .map(line => {
                const [ipRange, countryCode, stateCode, city, postalCode] = line.split(',');
                return { ipRange, countryCode, stateCode, city, postalCode };
            });
    }

    function populateCountrySelect(countries) {
        countrySelect.innerHTML = '<option value="">Select Country</option>';
        countries.sort((a, b) => a.name.localeCompare(b.name))
            .forEach(country => {
                const option = document.createElement('option');
                option.value = country.iso2;
                option.textContent = `${country.emoji} ${country.name}`;
                countrySelect.appendChild(option);
            });
    }

    function updateStateSelect(countryCode) {
        stateSelect.innerHTML = '<option value="">Select State</option>';
        if (!countryCode) return;

        const countryStates = cache.states.filter(state => state.country_code === countryCode);
        countryStates.sort((a, b) => a.name.localeCompare(b.name))
            .forEach(state => {
                const option = document.createElement('option');
                option.value = state.state_code;
                option.textContent = state.name;
                stateSelect.appendChild(option);
            });
    }

    function displayResults(results) {
        tableBody.innerHTML = '';
        results.forEach(result => {
            const tr = document.createElement('tr');
            tr.innerHTML = `
                <td>${result.ipRange}</td>
                <td>${result.countryCode}</td>
                <td>${result.stateCode}</td>
                <td>${result.city || '-'}</td>
                <td>${result.postalCode || '-'}</td>
            `;
            tableBody.appendChild(tr);
        });
    }

    function filterResults() {
        const searchTerm = searchInput.value.toLowerCase();
        const countryCode = countrySelect.value;
        const stateCode = stateSelect.value;

        let filtered = cache.ipData;

        if (countryCode) {
            filtered = filtered.filter(item => item.countryCode === countryCode);
        }

        if (stateCode) {
            filtered = filtered.filter(item => item.stateCode === stateCode);
        }

        if (searchTerm) {
            filtered = filtered.filter(item =>
                item.ipRange.toLowerCase().includes(searchTerm) ||
                item.city?.toLowerCase().includes(searchTerm) ||
                item.postalCode?.toLowerCase().includes(searchTerm)
            );
        }

        displayResults(filtered.slice(0, 50));
    }

    // Event Listeners
    countrySelect.addEventListener('change', () => {
        updateStateSelect(countrySelect.value);
        filterResults();
    });

    stateSelect.addEventListener('change', filterResults);

    searchInput.addEventListener('input', debounce(filterResults, 300));

    function debounce(func, wait) {
        let timeout;
        return function executedFunction(...args) {
            const later = () => {
                clearTimeout(timeout);
                func(...args);
            };
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
        };
    }

    // Initialize the application
    initializeData();
});